#!/usr/bin/env python3
# -*-coding:utf-8 -*-

"""
@File    :   util.py
@Date    :   2024-01-22
@Author  :   yuanci.ytb
@Version :   1.0.0
@License :   Copyright(C) 1999-2023, All rights Reserved, Designed By Alibaba Group Inc. 
@Desc    :   #TODO
"""
import warnings


def deprecated(func):
    def new_func(*args, **kwargs):
        warnings.simplefilter('always', DeprecationWarning)
        warnings.warn(f"function [{func.__name__}] is deprecated and will be removed in future versions.",
                      category=DeprecationWarning, stacklevel=2)
        warnings.simplefilter('default', DeprecationWarning)
        return func(*args, **kwargs)

    return new_func



import json
from typing import Dict, Optional, Iterator, List, Union, Tuple, Any

from broadscope_bailian.api import BaseCompletions
from broadscope_bailian.api.models import ChatQaMessage
from broadscope_bailian.api.models import ChatRequestQaMessage
from broadscope_bailian.api.models import ChatRequestMessage
from broadscope_bailian.api.util import deprecated


class Completions(BaseCompletions):
    """文本生成"""

    def create(self,
               app_id: str,
               prompt: str = None,
               request_id: str = None,
               session_id: str = None,
               history: List[ChatRequestQaMessage] = None,
               messages: List[ChatRequestMessage] = None,
               top_p: float = 0.0,
               stream: bool = False,
               top_k: int = None,
               seed: int = None,
               use_raw_prompt: bool = None,
               temperature: float = None,
               max_tokens: int = None,
               result_format: str = None,
               stop: List[str] = None,
               incremental_output: bool = None,
               doc_tag_codes: List[str] = None,
               doc_reference_type: str = None,
               biz_params: Dict = None,
               has_thoughts: bool = False,
               timeout: Union[float, Tuple[float, float]] = None):
        resp = self.reqeust(app_id=app_id,
                            prompt=prompt,
                            request_id=request_id,
                            session_id=session_id,
                            history=history,
                            messages=messages,
                            top_p=top_p,
                            stream=stream,
                            top_k=top_k,
                            seed=seed,
                            use_raw_prompt=use_raw_prompt,
                            temperature=temperature,
                            max_tokens=max_tokens,
                            result_format=result_format,
                            stop=stop,
                            incremental_output=incremental_output,
                            doc_reference_type=doc_reference_type,
                            doc_tag_codes=doc_tag_codes,
                            biz_params=biz_params,
                            has_thoughts=has_thoughts,
                            timeout=timeout)

        if stream:
            return (json.loads(line) for line in self.parse_response_stream(resp.iter_lines()))
        else:
            return json.loads(resp.text)

    @deprecated
    def call(self,
             app_id: str,
             prompt: str,
             request_id: str = None,
             session_id: str = None,
             history: List[ChatQaMessage] = None,
             top_p: float = 0.0,
             biz_params: Dict = None,
             has_thoughts: bool = False,
             stream: bool = False,
             doc_reference_type: str = None,
             top_k: int = None,
             seed: int = None,
             use_raw_prompt: bool = None,
             doc_tag_ids: List[int] = None,
             timeout: Union[float, Tuple[float, float]] = None,
             **kwargs: Any):

        temperature = kwargs.get("temperature", None)
        max_tokens = kwargs.get("max_tokens", None)
        result_format = kwargs.get("result_format", None)
        stop = kwargs.get("stop", None)
        incremental_output = kwargs.get("incremental_output", None)

        new_history = None
        if history is not None:
            new_history = []
            for h in history:
                new_history.append({"user": h.user, "bot": h.bot})

        resp = self.reqeust(app_id=app_id,
                            prompt=prompt,
                            request_id=request_id,
                            session_id=session_id,
                            history=new_history,
                            top_p=top_p,
                            stream=stream,
                            top_k=top_k,
                            seed=seed,
                            use_raw_prompt=use_raw_prompt,
                            temperature=temperature,
                            max_tokens=max_tokens,
                            result_format=result_format,
                            stop=stop,
                            incremental_output=incremental_output,
                            doc_reference_type=doc_reference_type,
                            doc_tag_ids=doc_tag_ids,
                            biz_params=biz_params,
                            has_thoughts=has_thoughts,
                            timeout=timeout)

        if stream:
            return (json.loads(line) for line in self.parse_response_stream(resp.iter_lines()))
        else:
            return json.loads(resp.text)

    def parse_response_stream(self, resp: Iterator[bytes]) -> Iterator[str]:
        for line in resp:
            _line = self.parse_stream_line(line)
            if _line is not None:
                yield _line

    @staticmethod
    def parse_stream_line(line: bytes) -> Optional[str]:
        if line:
            if line.strip() == b"data: [DONE]":
                return None
            if line.startswith(b"data: "):
                line = line[len(b"data: "):]
                return line.decode("utf-8")
            else:
                return None
        return None



from typing import Dict, Union

from typing_extensions import TypedDict, Required, Literal

from broadscope_bailian.api.util import deprecated


class BaseModel:
    def from_dict(self, data: Dict = None):
        pass

    def to_dict(self):
        pass


@deprecated
class ChatQaMessage(BaseModel):
    def __init__(self, user, bot):
        self.user = user
        self.bot = bot

    def from_dict(self, data: Dict = None):
        data = data or dict()
        if data.get("User") is not None:
            self.user = data.get("User")
        if data.get("Bot") is not None:
            self.bot = data.get("Bot")

        return self

    def to_dict(self):
        data = dict()
        if self.user is not None:
            data["User"] = self.user
        if self.bot is not None:
            data["Bot"] = self.bot

        return data


class ChatRequestQaMessage(TypedDict):
    user: Required[str]
    """ content of user """

    bot: Required[str]
    """ content of bot """


class ChatSystemMessage(TypedDict):
    role: Required[Literal["system"]]
    """ system role """

    content: str
    """ content of system """


class ChatUserMessage(TypedDict):
    role: Required[Literal["user"]]
    """ user role """

    content: Required[str]
    """ content of user """


class ChatAssistantMessage(TypedDict):
    role: Required[Literal["assistant"]]
    """ user role """

    content: Required[str]
    """ content of user """


ChatRequestMessage = Union[
    ChatSystemMessage,
    ChatUserMessage,
    ChatAssistantMessage
]



from broadscope_bailian.api.base_completions import BaseCompletions
from broadscope_bailian.api.completions import Completions
from broadscope_bailian.api.access_token_client import AccessTokenClient
from broadscope_bailian.api.util import deprecated

from broadscope_bailian.api.models import (
    ChatQaMessage,
    ChatRequestQaMessage,
    ChatRequestMessage,
    ChatSystemMessage,
    ChatUserMessage,
    ChatAssistantMessage
)



import os

from broadscope_bailian.api import (
    Completions,
    AccessTokenClient,
    ChatQaMessage,
    ChatRequestQaMessage,
    ChatSystemMessage,
    ChatUserMessage,
    ChatAssistantMessage,
    deprecated
)

api_key = os.environ.get("BROADSCOPE_API_KEY")
api_base = os.environ.get("BROADSCOPE_API_BASE")
pop_endpoint = "bailian.cn-beijing.aliyuncs.com"
api_version = "1.1.8"



import time
from typing import Tuple

from alibabacloud_bailian20230601.client import Client
from alibabacloud_bailian20230601.models import CreateTokenRequest
from alibabacloud_tea_openapi.models import Config

import broadscope_bailian


class AccessToken:
    def __init__(self, token: str, expired_time: int):
        self.token = token
        self.expired_time = expired_time


class AccessTokenClient:
    def __init__(self, access_key_id: str,
                 access_key_secret: str,
                 agent_key: str = None,
                 endpoint: str = None):
        if endpoint is None or endpoint == "":
            endpoint = broadscope_bailian.pop_endpoint

        self.config = Config(access_key_id=access_key_id,
                             access_key_secret=access_key_secret,
                             endpoint=endpoint)
        self.agent_key = agent_key
        self.access_token = None

    def create_token(self, agent_key) -> Tuple[str, int]:
        """
        create access token for access SFM API

        :param agent_key: agent key of SFM
        :return: token and expired time with unix format
        """

        client = Client(config=self.config)

        token_response = client.create_token(CreateTokenRequest(agent_key=agent_key))
        if token_response.status_code != 200 or token_response.body is None:
            raise RuntimeError("create token error, code=%d" % token_response.status_code)

        token_body = token_response.body
        if not token_body.success:
            request_id = token_body.request_id
            if not request_id:
                request_id = token_response.headers.get("x-acs-request-id")
            raise RuntimeError("create token error, code=%s, message=%s RequestId: %s"
                               % (token_body.code, token_body.message, request_id))

        return token_body.data.token, token_body.data.expired_time

    def get_token(self):
        """ get token if not created or expired """

        timestamp = int(time.time())
        if self.access_token is None or (self.access_token.expired_time - 600) < timestamp:
            token, expired_time = self.create_token(self.agent_key)
            self.access_token = AccessToken(token, expired_time)

        return self.access_token.token



import json
import threading
import uuid
from typing import Dict, Any, List, Union, Tuple

import requests

import broadscope_bailian
from broadscope_bailian.api.models import ChatRequestQaMessage
from broadscope_bailian.api.models import ChatRequestMessage

_thread_context = threading.local()


class CompletionsRequestError(Exception):
    def __init__(self,
                 message=None,
                 status_code=None,
                 response_body=None
                 ):
        super(CompletionsRequestError, self).__init__(message)

        self.message = message
        self.status_code = status_code
        self.response_body = response_body

    def __str__(self):
        return "%s: status_code=%s, response_body=%s" % (
            self.message,
            self.status_code,
            self.response_body
        )

    def __repr__(self):
        return "(message=%r, status_code=%r, response_body=%r)" % (
            self.message,
            self.status_code,
            self.response_body
        )


class BaseCompletions:
    """ 调用百炼进行文本生成 """

    def __init__(self, token=None, endpoint=None):
        self.token = token
        self.endpoint = endpoint

    @classmethod
    def call(cls, *args, **kwargs) -> Any:
        raise NotImplementedError()

    def reqeust(self, app_id: str,
                prompt: str,
                request_id: str = None,
                session_id: str = None,
                history: List[ChatRequestQaMessage] = None,
                messages: List[ChatRequestMessage] = None,
                top_p: float = 0.0,
                stream: bool = False,
                top_k: int = None,
                seed: int = None,
                use_raw_prompt: bool = None,
                temperature: float = None,
                max_tokens: int = None,
                result_format: str = None,
                stop: List[str] = None,
                incremental_output: bool = None,
                doc_reference_type: str = None,
                doc_tag_ids: List[int] = None,
                doc_tag_codes: List[str] = None,
                biz_params: Dict = None,
                has_thoughts: bool = False,
                timeout: Union[float, Tuple[float, float]] = None):

        self.validate(app_id=app_id, prompt=prompt, messages=messages)

        headers = dict()
        headers["Content-Type"] = "application/json;charset=UTF-8"
        headers["Authorization"] = "Bearer %s" % self.token

        if stream:
            headers["Accept"] = "text/event-stream"

        if request_id is None:
            uuid_obj = uuid.uuid4()
            request_id = str(uuid_obj).replace('-', '')

        parameters = {}
        if top_k is not None:
            parameters["TopK"] = top_k
        if seed is not None:
            parameters["Seed"] = seed
        if use_raw_prompt is not None:
            parameters["UseRawPrompt"] = use_raw_prompt
        if temperature is not None:
            parameters["Temperature"] = temperature
        if max_tokens is not None:
            parameters["MaxTokens"] = max_tokens
        if result_format is not None:
            parameters["ResultFormat"] = result_format
        if stop is not None:
            parameters["Stop"] = stop
        if incremental_output is not None:
            parameters["IncrementalOutput"] = incremental_output

        data = {
            "RequestId": request_id,
            "SessionId": session_id,
            "AppId": app_id,
            "Prompt": prompt,
            "TopP": top_p,
            "Stream": stream,
            "HasThoughts": has_thoughts,
            "BizParams": biz_params,
            "DocReferenceType": doc_reference_type,
            "Parameters": parameters,
            "DocTagIds": doc_tag_ids,
            "DocTagCodes": doc_tag_codes,
        }

        if history is not None:
            new_history = []
            for v in history:
                new_history.append({"User": v.get("user"), "Bot": v.get("bot")})

            data["History"] = new_history

        if messages is not None:
            new_messages = []
            for m in messages:
                new_messages.append({"Role": m.get("role"), "Content": m.get("content")})

            data["Messages"] = new_messages

        url = "%s%s" % (self.endpoint, "/v2/app/completions")
        session = self.__get_session()
        resp = session.request("POST",
                               url=url,
                               headers=headers,
                               data=json.dumps(data),
                               stream=stream,
                               timeout=timeout)

        if not resp.ok:
            raise CompletionsRequestError("completion request error", resp.status_code, resp.text)

        return resp

    @staticmethod
    def __get_session() -> requests.Session:
        if not hasattr(_thread_context, "session"):
            _thread_context.session = requests.Session()

        return _thread_context.session

    def validate(self, app_id: str,
                 prompt: str, messages: List[dict]):
        if self.token is None:
            self.token = broadscope_bailian.api_key

        if broadscope_bailian.api_base is None:
            broadscope_bailian.api_base = "https://bailian.aliyuncs.com"

        if self.endpoint is None:
            self.endpoint = broadscope_bailian.api_base

        if self.token is None:
            raise ValueError("you need to set token before calling api")

        if app_id is None or app_id == '':
            raise ValueError("app id is required")

        if (prompt is None or prompt == '') and (messages is None or len(messages) == 0):
            raise ValueError("prompt or message is required")

    @staticmethod
    def get_param(key, cls_type, **kwargs):
        val = kwargs.get(key, None)
        if val is None:
            return None

        if not isinstance(val, cls_type):
            raise TypeError("%s is not instance of %s" % (key, cls_type))

        return val



from setuptools import setup, find_packages

PACKAGE = "broadscope_bailian"
VERSION = "1.3.0"

requires = [
    "requests >= 2.20",
    "alibabacloud_bailian20230601 >= 1.1.2"
    "typing-extensions>=4.7.0, <5"
]

setup(
    name=PACKAGE,
    version=VERSION,
    description="Alibaba Cloud Broadscope BaiLian SDK for Python",
    author="Alibaba Cloud SDK",
    author_email="sdk-team@alibabacloud.com",
    url="https://bailian.console.aliyun.com/",
    keywords=["alibabacloud", "broadscope-bailian"],
    packages=find_packages(exclude=["tests*"]),
    platforms="any",
    install_requires=requires,
    python_requires=">=3.7",
    license="MIT License",
    classifiers=[
        "Intended Audience :: Developers",
        "Natural Language :: English",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
    ],
)
